@@ -21,11 +21,10 @@
 # with this program; if not, see <http://www.gnu.org/licenses/>.
 #
 #
-import os
-import os.path
 import subprocess
 from datetime import datetime, timedelta
 from functools import partial
+from os import path
 
 from qubesadmin import exc
 from qubesadmin import utils
@@ -37,7 +36,7 @@ from PyQt5.QtCore import (Qt, QAbstractTableModel, QObject, pyqtSlot, QEvent,
 # pylint: disable=import-error
 from PyQt5.QtWidgets import (QLineEdit, QStyledItemDelegate, QToolTip,
     QMenu, QInputDialog, QMainWindow, QProgressDialog, QStyleOptionViewItem,
-    QAbstractItemView, QMessageBox)
+    QMessageBox)
 
 # pylint: disable=import-error
 from PyQt5.QtGui import (QIcon, QPixmap, QRegExpValidator, QFont, QColor)
@@ -360,7 +359,7 @@ class QubesTableModel(QAbstractTableModel):
                 "Disk Usage",
                 "Internal",
                 "IP",
-                "Include in backups",
+                "Backup",
                 "Last backup",
                 "Default DispVM",
                 "Is DVM Template",
@@ -405,8 +404,6 @@ class QubesTableModel(QAbstractTableModel):
                 return "Yes" if vm.internal else ""
             if col_name == "IP":
                 return vm.ip
-            if col_name == "Include in backups":
-                return "Yes" if vm.inc_backup else ""
             if col_name == "Last backup":
                 return vm.last_backup
             if col_name == "Default DispVM":
@@ -429,7 +426,6 @@ class QubesTableModel(QAbstractTableModel):
                     return self.klass_pixmap[vm.klass]
                 except exc.QubesDaemonAccessError:
                     return None
-
             if col_name == "Label":
                 try:
                     return self.label_pixmap[vm.icon]
@@ -437,23 +433,22 @@ class QubesTableModel(QAbstractTableModel):
                     icon = QIcon.fromTheme(vm.icon)
                     self.label_pixmap[vm.icon] = icon.pixmap(icon_size)
                     return self.label_pixmap[vm.icon]
-
+        if role == Qt.CheckStateRole:
+            if col_name == "Backup":
+                return Qt.Checked if vm.inc_backup else Qt.Unchecked
         if role == Qt.FontRole:
             if col_name == "Template":
                 if vm.template is None:
                     font = QFont()
                     font.setItalic(True)
                     return font
-
         if role == Qt.ForegroundRole:
             if col_name == "Template":
                 if vm.template is None:
                     return QColor("gray")
-
         # Used for get VM Object
         if role == Qt.UserRole:
             return vm
-
         # Used for sorting
         if role == Qt.UserRole + 1:
             if vm.klass == 'AdminVM':
@@ -466,7 +461,6 @@ class QubesTableModel(QAbstractTableModel):
                 return str(vm.state)
             if col_name == "Disk Usage":
                 return vm.disk_float
-
             return self.data(index, Qt.DisplayRole)
 
     # pylint: disable=invalid-name
@@ -477,6 +471,27 @@ class QubesTableModel(QAbstractTableModel):
             return self.columns_indices[col]
         return None
 
+    def setData(self, index, value, role=Qt.EditRole):
+        if not index.isValid():
+            return False
+
+        if role == Qt.CheckStateRole:
+            col_name = self.columns_indices[index.column()]
+            if col_name == "Backup":
+                vm = self.qubes_cache.get_vm(index.row())
+                vm.vm.include_in_backups = (value == Qt.Checked)
+                vm.inc_backup = (value == Qt.Checked)
+                return True
+        return False
+
+    def flags(self, index):
+        if not index.isValid():
+            return False
+
+        def_flags = QAbstractTableModel.flags(self, index)
+        if self.columns_indices[index.column()] == "Backup":
+            return  def_flags | Qt.ItemIsUserCheckable
+        return def_flags
 
 vm_shutdown_timeout = 20000  # in msec
 vm_restart_check_timeout = 1000  # in msec
@@ -533,7 +548,14 @@ class VmShutdownMonitor(QObject):
                     self.tr("Wait another {0} seconds...").format(
                         self.shutdown_time / 1000),
                     QMessageBox.NoRole)
+                ignore_button = msgbox.addButton(self.tr("Don't ask again"),
+                                                 QMessageBox.RejectRole)
                 msgbox.setDefaultButton(wait_button)
+                msgbox.setEscapeButton(ignore_button)
+                msgbox.setWindowFlags(
+                    msgbox.windowFlags() | Qt.CustomizeWindowHint)
+                msgbox.setWindowFlags(
+                    msgbox.windowFlags() & ~Qt.WindowCloseButtonHint)
                 msgbox.exec_()
                 msgbox.deleteLater()
 
@@ -545,6 +567,8 @@ class VmShutdownMonitor(QObject):
                         # shutting it down
                         pass
                     self.restart_vm_if_needed()
+                elif msgbox.clickedButton() is ignore_button:
+                    return
                 else:
                     self.shutdown_started = datetime.now()
                     self.check_again_later()
@@ -621,8 +645,11 @@ class RunCommandThread(common_threads.QubesThread):
         except (ChildProcessError, exc.QubesException) as ex:
             self.msg = (self.tr("Error while running command!"), str(ex))
 
-
 class QubesProxyModel(QSortFilterProxyModel):
+    def __init__(self, window):
+        super().__init__()
+        self.window = window
+
     def lessThan(self, left, right):
         if left.data(self.sortRole()) != right.data(self.sortRole()):
             return super().lessThan(left, right)
@@ -632,6 +659,31 @@ class QubesProxyModel(QSortFilterProxyModel):
 
         return left_vm.name.lower() < right_vm.name.lower()
 
+    # pylint: disable=too-many-return-statements
+    def filterAcceptsRow(self, sourceRow, sourceParent):
+        if self.window.show_all.isChecked():
+            return super().filterAcceptsRow(sourceRow, sourceParent)
+
+        index = self.sourceModel().index(sourceRow, 0, sourceParent)
+        vm = self.sourceModel().data(index, Qt.UserRole)
+
+        if self.window.show_running.isChecked() and \
+                vm.state['power'] == 'Running':
+            return super().filterAcceptsRow(sourceRow, sourceParent)
+        if self.window.show_halted.isChecked() and \
+                vm.state['power'] == 'Halted':
+            return super().filterAcceptsRow(sourceRow, sourceParent)
+        if self.window.show_network.isChecked() and \
+                getattr(vm.vm, 'provides_network', False):
+            return super().filterAcceptsRow(sourceRow, sourceParent)
+        if self.window.show_templates.isChecked() and vm.klass == 'TemplateVM':
+            return super().filterAcceptsRow(sourceRow, sourceParent)
+        if self.window.show_standalone.isChecked() \
+                and vm.klass == 'StandaloneVM':
+            return super().filterAcceptsRow(sourceRow, sourceParent)
+
+        return False
+
 
 class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
     # suppress saving settings while initializing widgets
@@ -650,38 +702,14 @@ class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
         self.searchbox.setValidator(QRegExpValidator(
             QRegExp("[a-zA-Z0-9_-]*", Qt.CaseInsensitive), None))
         self.searchbox.textChanged.connect(self.do_search)
-        self.searchContainer.addWidget(self.searchbox)
+        self.searchContainer.insertWidget(1, self.searchbox)
 
         self.settings_windows = {}
 
         self.frame_width = 0
         self.frame_height = 0
 
-        self.context_menu = QMenu(self)
-
-        self.context_menu.addAction(self.action_settings)
-        self.context_menu.addAction(self.action_editfwrules)
-        self.context_menu.addAction(self.action_appmenus)
-        self.context_menu.addAction(self.action_set_keyboard_layout)
-        self.context_menu.addSeparator()
-
-        self.context_menu.addAction(self.action_updatevm)
-        self.context_menu.addAction(self.action_run_command_in_vm)
-        self.context_menu.addAction(self.action_open_console)
-        self.context_menu.addAction(self.action_resumevm)
-        self.context_menu.addAction(self.action_startvm_tools_install)
-        self.context_menu.addAction(self.action_pausevm)
-        self.context_menu.addAction(self.action_shutdownvm)
-        self.context_menu.addAction(self.action_restartvm)
-        self.context_menu.addAction(self.action_killvm)
-        self.context_menu.addSeparator()
-
-        self.context_menu.addAction(self.action_clonevm)
-        self.context_menu.addAction(self.action_removevm)
-        self.context_menu.addSeparator()
-
-        self.context_menu.addMenu(self.logs_menu)
-        self.context_menu.addSeparator()
+        self.__init_context_menu()
 
         self.tools_context_menu = QMenu(self)
         self.tools_context_menu.addAction(self.action_toolbar)
@@ -693,7 +721,8 @@ class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
                 lambda pos: self.open_tools_context_menu(self.toolbar, pos))
         self.action_menubar.toggled.connect(self.showhide_menubar)
         self.action_toolbar.toggled.connect(self.showhide_toolbar)
-        self.logs_menu.triggered.connect(self.show_log)
+        self.action_show_logs.triggered.connect(self.show_log)
+        self.action_compact_view.toggled.connect(self.set_compactview)
 
         self.table.resizeColumnsToContents()
 
@@ -704,7 +733,7 @@ class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
         self.fill_cache()
         self.qubes_model = QubesTableModel(self.qubes_cache)
 
-        self.proxy = QubesProxyModel()
+        self.proxy = QubesProxyModel(self)
         self.proxy.setSourceModel(self.qubes_model)
         self.proxy.setSortRole(Qt.UserRole + 1)
         self.proxy.setSortCaseSensitivity(Qt.CaseInsensitive)
@@ -712,10 +741,16 @@ class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
         self.proxy.setFilterCaseSensitivity(Qt.CaseInsensitive)
         self.proxy.layoutChanged.connect(self.save_sorting)
 
+        self.show_running.stateChanged.connect(self.invalidate)
+        self.show_halted.stateChanged.connect(self.invalidate)
+        self.show_network.stateChanged.connect(self.invalidate)
+        self.show_templates.stateChanged.connect(self.invalidate)
+        self.show_standalone.stateChanged.connect(self.invalidate)
+        self.show_all.stateChanged.connect(self.invalidate)
+
         self.table.setModel(self.proxy)
         self.table.setItemDelegateForColumn(3, StateIconDelegate())
         self.table.resizeColumnsToContents()
-        self.table.setSelectionMode(QAbstractItemView.ExtendedSelection)
         selection_model = self.table.selectionModel()
         selection_model.selectionChanged.connect(self.table_selection_changed)
 
@@ -733,6 +768,8 @@ class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
         self.menu_view.addSeparator()
         self.menu_view.addAction(self.action_toolbar)
         self.menu_view.addAction(self.action_menubar)
+        self.menu_view.addSeparator()
+        self.menu_view.addAction(self.action_compact_view)
 
         try:
             self.load_manager_settings()
@@ -785,12 +822,51 @@ class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
 
         self.check_updates()
 
+    def __init_context_menu(self):
+        self.context_menu = QMenu(self)
+        self.context_menu.addAction(self.action_settings)
+        self.context_menu.addAction(self.action_editfwrules)
+        self.context_menu.addAction(self.action_appmenus)
+        self.context_menu.addAction(self.action_set_keyboard_layout)
+        self.context_menu.addSeparator()
+        self.context_menu.addAction(self.action_updatevm)
+        self.context_menu.addAction(self.action_run_command_in_vm)
+        self.context_menu.addAction(self.action_open_console)
+        self.context_menu.addAction(self.action_resumevm)
+        self.context_menu.addAction(self.action_startvm_tools_install)
+        self.context_menu.addAction(self.action_pausevm)
+        self.context_menu.addAction(self.action_shutdownvm)
+        self.context_menu.addAction(self.action_restartvm)
+        self.context_menu.addAction(self.action_killvm)
+        self.context_menu.addSeparator()
+        self.context_menu.addAction(self.action_clonevm)
+        self.context_menu.addAction(self.action_removevm)
+        self.context_menu.addSeparator()
+        self.context_menu.addAction(self.action_show_logs)
+
+    def save_showing(self):
+        self.manager_settings.setValue('show/running',
+                self.show_running.isChecked())
+        self.manager_settings.setValue('show/halted',
+                self.show_halted.isChecked())
+        self.manager_settings.setValue('show/network',
+                self.show_network.isChecked())
+        self.manager_settings.setValue('show/templates',
+                self.show_templates.isChecked())
+        self.manager_settings.setValue('show/standalone',
+                self.show_standalone.isChecked())
+        self.manager_settings.setValue('show/all', self.show_all.isChecked())
+
     def save_sorting(self):
         self.manager_settings.setValue('view/sort_column',
                 self.proxy.sortColumn())
         self.manager_settings.setValue('view/sort_order',
                 self.proxy.sortOrder())
 
+    def invalidate(self):
+        self.proxy.invalidate()
+        self.table.resizeColumnsToContents()
+
     def fill_cache(self):
         progress = QProgressDialog(
             self.tr(
@@ -938,6 +1014,23 @@ class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
         if not self.manager_settings.value("view/toolbar_visible",
                                            defaultValue=True):
             self.action_toolbar.setChecked(False)
+        if self.manager_settings.value("view/compactview",
+                                           defaultValue="false") != "false":
+            self.action_compact_view.setChecked(True)
+
+        # Restore show checkboxes
+        self.show_running.setChecked(self.manager_settings.value(
+            'show/running', "true") == "true")
+        self.show_halted.setChecked(self.manager_settings.value(
+            'show/halted', "true") == "true")
+        self.show_network.setChecked(self.manager_settings.value(
+            'show/network', "true") == "true")
+        self.show_templates.setChecked(self.manager_settings.value(
+            'show/templates', "true") == "true")
+        self.show_standalone.setChecked(self.manager_settings.value(
+            'show/standalone', "true") == "true")
+        self.show_all.setChecked(self.manager_settings.value(
+            'show/all', "true") == "true")
 
         # load last window size
         self.resize(self.manager_settings.value("window_size",
@@ -1021,8 +1114,6 @@ class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
             if not vm.updateable and vm.klass != 'AdminVM':
                 self.action_updatevm.setEnabled(False)
 
-        self.update_logs_menu()
-
     # noinspection PyArgumentList
     @pyqtSlot(name='on_action_createvm_triggered')
     def action_createvm_triggered(self):
@@ -1255,6 +1346,9 @@ class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
                     "\nError: {}".format(str(ex))))
             return
 
+    def closeEvent(self, _):
+        self.save_showing()
+
     # noinspection PyArgumentList
     @pyqtSlot(name='on_action_settings_triggered')
     def action_settings_triggered(self):
@@ -1381,6 +1475,14 @@ class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
     def action_exit_triggered(self):
         self.close()
 
+    def set_compactview(self, checked):
+        if checked:
+            self.toolbar.setToolButtonStyle(Qt.ToolButtonIconOnly)
+        else:
+            self.toolbar.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
+        if self.settings_loaded:
+            self.manager_settings.setValue('view/compactview', checked)
+
     def showhide_menubar(self, checked):
         self.menubar.setVisible(checked)
         if not checked:
@@ -1419,48 +1521,42 @@ class VmManagerWindow(ui_qubemanager.Ui_VmManagerWindow, QMainWindow):
     def open_tools_context_menu(self, widget, point):
         self.tools_context_menu.exec_(widget.mapToGlobal(point))
 
-    def update_logs_menu(self):
-        self.logs_menu.clear()
-        menu_empty = True
-
-        try:
-            vm_info = self.get_selected_vms()
-
-            if len(vm_info) == 1:
-                vm = vm_info[0].vm
-
-                if vm.klass == 'AdminVM':
-                    logfiles = ["/var/log/xen/console/hypervisor.log"]
-                else:
-                    logfiles = [
-                        "/var/log/xen/console/guest-" + vm.name + ".log",
-                        "/var/log/xen/console/guest-" + vm.name + "-dm.log",
-                        "/var/log/qubes/guid." + vm.name + ".log",
-                        "/var/log/qubes/qrexec." + vm.name + ".log",
-                    ]
-
-                for logfile in logfiles:
-                    if os.path.exists(logfile):
-                        action = self.logs_menu.addAction(QIcon(":/log.png"),
-                                                          logfile)
-                        action.setData(logfile)
-                        menu_empty = False
-
-            self.logs_menu.setEnabled(not menu_empty)
-        except exc.QubesDaemonAccessError:
-            pass
-
     @pyqtSlot('const QPoint&')
     def open_context_menu(self, point):
         self.context_menu.exec_(self.table.mapToGlobal(
             point + QPoint(10, 0)))
 
-    @pyqtSlot('QAction *')
-    def show_log(self, action):
-        log = str(action.data())
-        log_dlg = log_dialog.LogDialog(self.qt_app, log)
-        log_dlg.exec_()
+    def show_log(self):
+        logfiles = []
 
+        try:
+            for vm_info in self.get_selected_vms():
+                vm = vm_info.vm
+
+                if vm.klass == 'AdminVM':
+                    logfiles.append("/var/log/xen/console/hypervisor.log")
+                else:
+                    logfiles.extend([
+                        "/var/log/xen/console/guest-" + vm.name + ".log",
+                        "/var/log/xen/console/guest-" + vm.name + "-dm.log",
+                        "/var/log/qubes/guid." + vm.name + ".log",
+                        "/var/log/qubes/qrexec." + vm.name + ".log",
+                    ])
+
+            logfiles = [x for x in logfiles if path.exists(x)]
+
+            if len(logfiles) > 0:
+                log_dlg = log_dialog.LogDialog(self.qt_app, logfiles)
+                log_dlg.exec_()
+            else:
+                QMessageBox.warning(
+                    self,
+                    self.tr("Error"),
+                    self.tr(
+                        "No log files where found for the current selection."))
+
+        except exc.QubesDaemonAccessError:
+            pass
 
 def main():
     manager_utils.run_asynchronous(VmManagerWindow)
