@@ -1,7 +1,7 @@
 import asyncio
 import sys
 
-import quamash
+import qasync
 from PyQt5 import QtWidgets
 
 qtapp = None
@@ -11,7 +11,7 @@ def init_qtapp():
     global qtapp, loop
     if qtapp is None:
         qtapp = QtWidgets.QApplication(sys.argv)
-        loop = quamash.QEventLoop(qtapp)
+        loop = qasync.QEventLoop(qtapp)
         asyncio.set_event_loop(loop)
     qtapp.processEvents()
     return qtapp, loop
